use actix_web::{web, HttpResponse, Responder};
use askama::Template;
use askama_actix::TemplateToResponse;
use chrono::NaiveDate;
use sqlx::PgPool;

use crate::{
    endpoints::{IdPath, NaiveDateQuery},
    models::{Assignment, Availabillity, Event, Function, Location, Role, User},
    utils::ApplicationError,
};

#[derive(Template)]
#[template(path = "events/plan.html")]
pub struct PlanEventTemplate {
    user: User,
    event: Event,
    availabillities: Vec<(Availabillity, AssignmentState)>,
}

enum AssignmentState {
    // availabillity is not assigned at all or at least not timely conflicting
    Unassigned,
    // availabillity is assigned for another event that is timely conflicting
    Conflicting,
    // availabillity is assigned to this event as Posten
    AssignedPosten,
    // availabillity is assigned to this event as Führungsassistent
    AssignedFührungsassistent,
    // availabillity is assigned to this event as Wachhabender
    AssignedWachahabender,
}

#[actix_web::get("/events/{id}/plan")]
pub async fn get(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
) -> Result<impl Responder, ApplicationError> {
    if user.role != Role::Admin && user.role != Role::AreaManager {
        return Err(ApplicationError::Unauthorized);
    }

    let Some(event) = Event::read_by_id_including_location(pool.get_ref(), path.id).await? else {
        return Ok(HttpResponse::NotFound().finish());
    };

    if user.role != Role::Admin && user.area_id != event.location.as_ref().unwrap().area_id {
        return Err(ApplicationError::Unauthorized);
    }

    let availabillities_in_db = Availabillity::read_by_date_and_area_including_user(
        pool.get_ref(),
        event.date,
        event.location.as_ref().unwrap().area_id,
    )
    .await?;

    let mut availabillities = Vec::new();
    for availabillity in availabillities_in_db {
        let assignments =
            Assignment::read_all_by_availabillity(pool.get_ref(), availabillity.id).await?;

        if let Some(assignment) = assignments
            .iter()
            .find(|assignment| assignment.event_id == event.id)
        {
            let state = match assignment.function {
                Function::Posten => AssignmentState::AssignedPosten,
                Function::Fuehrungsassistent => AssignmentState::AssignedFührungsassistent,
                Function::Wachhabender => AssignmentState::AssignedWachahabender,
            };

            availabillities.push((availabillity, state));
            continue;
        }

        let has_start_time_during_event =
            |a: &Assignment| a.start_time >= event.start_time && a.start_time <= event.end_time;
        let has_end_time_during_event =
            |a: &Assignment| a.end_time >= event.start_time && a.end_time <= event.end_time;

        if assignments
            .iter()
            .any(|a| has_start_time_during_event(a) || has_end_time_during_event(a))
        {
            availabillities.push((availabillity, AssignmentState::Conflicting));
            continue;
        }

        availabillities.push((availabillity, AssignmentState::Unassigned));
    }

    let template = PlanEventTemplate {
        user: user.into_inner(),
        event,
        availabillities,
    };

    Ok(template.to_response())
}
